package android.aLlum;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.util.ArrayList;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class BDComandes {

	private Context 				context;
    private BDComandesHelper 		dbHelper;
    private static SQLiteDatabase 	db;
	
    private String dbName    = "DBaLlum";
	private int    dbVersion = 1;

	private static final String URL_VERSIO  = "http://192.168.1.38/versio.php";
	private static final String URL_CATALEG = "http://192.168.1.38/productes.php";

	private static final String TABLE_VER = "VersioCataleg";
	private static final String VER_DATA  = "dVersion";

	private static final String TABLE_COM = "Comandes";
	private static final String COM_ID    = "_id";
	private static final String COM_NOM   = "nom";

	private static final String TABLE_PRO = "Productes";
	private static final String PRO_ID    = "_id";
	private static final String PRO_NOM   = "nom";
	private static final String PRO_DES   = "desc";
	private static final String PRO_VIG   = "vigent";

	private static final String TABLE_PROD_COM = "ProdCom";
	private static final String PROCOM_IDCOM   = "idCom";
	private static final String PROCOM_IDPROD  = "idProd";
	private static final String PROCOM_UNITATS = "unitats";
	

    private static BDComandes 		dbCom = null;
 
    private class BDComandesHelper extends SQLiteOpenHelper {

    	BDComandesHelper() {
        	super(context, dbName, null, dbVersion);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
    		Log.i("BDComandesHelper", "BDComandes - OnCreate");
        	try {		
	        	// creaci de les taules
        		createTableComandes(db);
        		createTableProductes(db);
        		createTableProdCom(db);
        		createTableVersio(db);
        		
        		// Inicialitzar el catleg de Productes
        		String sData = obtenirDataVersioServidor();
        		Log.i("BDComandesHelper", "sData servidor=[" + sData + "]");
        		ArrayList<Producte> aProd;
        		aProd = obtenirNouCataleg();
        		inicialitzarCataleg(db, aProd, sData);
        		
    		} catch (Exception e) {
    			e.printStackTrace();
    		}
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        	Log.i("BDComandesHelper", "Update BD de la versi " + oldVersion + " a la versi " + newVersion);
        }
        
        private void createTableComandes(SQLiteDatabase db) {
        	String sCons = "CREATE TABLE " + TABLE_COM + " ( " +
        					COM_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
        					COM_NOM + " VARCHAR(30) NOT NULL UNIQUE)";
        	Log.i("BDComandesHelper", "createTableComandes: [" + sCons + "]");
        	db.execSQL(sCons);
        }

        private void createTableProductes(SQLiteDatabase db) {
        	String sCons = "CREATE TABLE " + TABLE_PRO + " ( " +
		        			PRO_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
		        			PRO_NOM + " VARCHAR(30) NOT NULL UNIQUE, " + 
		        			PRO_DES + " VARCHAR(150), " +
		        			PRO_VIG + " BOOLEAN DEFAULT '1');";
        	Log.i("BDComandesHelper", "createTableProductes: [" + sCons + "]");
        	db.execSQL(sCons);     	
        }

        private void createTableProdCom(SQLiteDatabase db) {
        	String sCons = "CREATE TABLE " + TABLE_PROD_COM + " ( " +
        			PROCOM_IDCOM   + " INTEGER, " +
        			PROCOM_IDPROD  + " INTEGER, " +
        			PROCOM_UNITATS + " INTEGER, " +
        			"PRIMARY KEY (" + PROCOM_IDCOM + ", " + PROCOM_IDPROD + "), " +
        			"FOREIGN KEY(" + PROCOM_IDCOM + ") REFERENCES " + TABLE_COM + " (" + COM_ID + "), " +
        			"FOREIGN KEY(" + PROCOM_IDPROD + ") REFERENCES " + TABLE_PRO + " (" + PRO_ID + "));";
        	Log.i("BDComandesHelper", "createTableProdCom: [" + sCons + "]");
        	db.execSQL(sCons);
        }     
        
        private void createTableVersio(SQLiteDatabase db) {
        	String sCons = "CREATE TABLE " + TABLE_VER + " ( " + VER_DATA   + " TIMESTAMP );";
        	Log.i("BDComandesHelper", "createTableProdCom: [" + sCons + "]");

        	// Creaci de la taula
        	db.execSQL(sCons);
        }        

        private void inicialitzarCataleg(SQLiteDatabase db, ArrayList<Producte> aProd, String sData) {
        	ContentValues    cv;
        	Producte        prod;
        	
        	for (int iPos=0; iPos < aProd.size(); iPos++) {
        		prod = aProd.get(iPos);
	        	cv = new ContentValues();
	    		cv.put(PRO_NOM, prod.getNom());
	    		cv.put(PRO_DES, prod.getDescripcio());
	    		db.insert(TABLE_PRO, null, cv);
        	}
        	
        	// Insertem la data d'actualitzaci del catleg
			cv = new ContentValues();
			cv.put(VER_DATA, sData);
			
			db.insert(TABLE_VER, null, cv);	
        } 
    }
    
    
    public static SQLiteDatabase getDB() {
    	return db;
    }
    
    public static BDComandes getInstance() {
    	if (dbCom == null) {
    		dbCom = new BDComandes();
    	}
    	return dbCom;
    }
    
	private BDComandes () {
	}
	
	public void open(Context context) {
		this.context = context;			
		dbHelper = new BDComandesHelper();
		db = dbHelper.getWritableDatabase();        
	}
	
    public void close() {
    	dbHelper.close();
    }
    
    public Comanda obtenirComanda(int idCom) {
		Cursor c = db.rawQuery("SELECT * FROM " + TABLE_COM + " WHERE _id =" + idCom, new String [] {});
		if (c.moveToFirst()) {
			Comanda com = new Comanda();
			com.setId(c.getInt(0));
			com.setNom(c.getString(1));
			return com;
		}
		else {
			return null;
		}
		
	}
	
	public ArrayList<Comanda> obtenirLlistaComandes() {

		ArrayList<Comanda> llistaCom = new ArrayList<Comanda>();
		
		Cursor c = db.query(TABLE_COM, new String[] {COM_ID, COM_NOM}, null, null, null, null, COM_ID + " desc");
		
		if (c!= null) {
			c.moveToFirst();	
			
			while (c.isAfterLast() == false) {
				
				Comanda com = new Comanda();
				com.setId(c.getInt(0));
				com.setNom(c.getString(1));
				llistaCom.add(com);
				c.moveToNext();
			}
		}
		return llistaCom;
	}
	
	public ArrayList<Producte> obtenirLlistaProductes(int idCom) {

		ArrayList<Producte> llistaProd = new ArrayList<Producte>();
		String  sCons;

		if (idCom > 0) {
			// Si es una comanda existens cal obtenir tots els productes i les
			// unitats que t la comanda de cada producte
			sCons = " SELECT " + PRO_ID + ", " + PRO_NOM + ", " + PRO_DES + ", " + PRO_VIG + ", " + PROCOM_UNITATS +
					" FROM " + TABLE_PRO + " LEFT OUTER JOIN " + TABLE_PROD_COM + " ON " +
					PRO_ID + " = " + PROCOM_IDPROD + " and " + PROCOM_IDCOM + " = " + idCom;
		} else {
			sCons = " SELECT " + PRO_ID + ", " + PRO_NOM + ", " + PRO_DES + ", " + PRO_VIG + ", 0 FROM " + TABLE_PRO;
		}
		
		Cursor c = db.rawQuery(sCons , new String [] {});
		
		if (c!= null) {
			c.moveToFirst();	
			
			while (c.isAfterLast() == false) {
				
				Producte prod = new Producte();
				prod.setId(c.getInt(0));
				prod.setNom(c.getString(1));
				prod.setDescripcio(c.getString(2));
				if (c.getInt(3) == 1) {
					prod.setVigent(true);
				} else {
					prod.setVigent(false);
				}
				prod.setUnitats(c.getInt(4));
				llistaProd.add(prod);
				c.moveToNext();
			}
		}
	
		return llistaProd;
	}

	public void actualitzarComanda(int idCom, String sNom) {
		ContentValues cv=new ContentValues();
		cv.put(COM_NOM, sNom);
		db.update(TABLE_COM, cv, COM_ID+"=?", new String []{String.valueOf(idCom)});
	}
	
	public void actualitzarProdCom(int idCom, ArrayList<Producte> lProd) {
		db.beginTransaction();
		try {
			db.delete(TABLE_PROD_COM, PROCOM_IDCOM+"=?",  new String [] {String.valueOf(idCom)});
			int iPos;
			for (iPos = 0; iPos < lProd.size(); iPos++) {
				if (lProd.get(iPos).getUnitats() > 0) {
					ContentValues cv = new ContentValues();
					cv.put(PROCOM_IDCOM, idCom);
					cv.put(PROCOM_IDPROD, lProd.get(iPos).getId());
					cv.put(PROCOM_UNITATS, lProd.get(iPos).getUnitats());
					db.insert(TABLE_PROD_COM, null, cv);				
				}
			}
			db.setTransactionSuccessful();
		} finally {
			db.endTransaction();
		}
	}
	
	public int insertarComanda(Comanda com) {
		int idCom;
		ContentValues cv = new ContentValues();
		cv.put(COM_NOM, com.getNom());
		idCom = (int)db.insert(TABLE_COM, null, cv);
		actualitzarProdCom(idCom, com.getLlistaProd());
		return idCom;
	}
   
    public String obtenirDataVersioServidor() {
    	String sData = "";
    	
    	try{
    		HttpClient 	httpclient = new DefaultHttpClient();
    	    HttpPost 	httppost   = new HttpPost(URL_VERSIO);
    	    HttpResponse response  = httpclient.execute(httppost);
    	    HttpEntity entity      = response.getEntity();
    	    InputStream is;
    	    
    	    if (entity != null) {
    	    	is = entity.getContent();
     	    	JSONObject jsonResp = new JSONObject(convertStreamToString(is));
     	    	sData = jsonResp.getString("versio");
    	    }
    	}catch(Exception e){
    	        Log.e("log_tag", "Error in http connection "+e.toString());
    	}
    	
    	return sData;
    }

    public String obtenirDataVersioLocal() {
    	String sData = "";
    	
    	Cursor c = db.rawQuery("SELECT * FROM " + TABLE_VER, null);
		if (c.moveToFirst()) {
			sData = c.getString(0);
		}
		return sData;
    }

    public void actualitzarCataleg(String sDataAct) {
    	ArrayList<Producte> lProd;
    	ContentValues        cv;
    	
    	lProd = obtenirNouCataleg();
    	
    	db.beginTransaction();
		try {
			// Esborrem les taules
			db.delete(TABLE_PROD_COM, null,  new String [] {});
			db.delete(TABLE_PRO, null,  new String [] {});
			db.delete(TABLE_COM, null,  new String [] {});
			db.delete(TABLE_VER, null,  new String [] {});
			
			// Insertem el nou catleg
			int iPos;
			for (iPos = 0; iPos < lProd.size(); iPos++) {
				cv = new ContentValues();
				cv.put(PRO_NOM, lProd.get(iPos).getNom());
				cv.put(PRO_DES, lProd.get(iPos).getDescripcio());
				db.insert(TABLE_PRO, null, cv);				
			}
			
			// Insertem la data d'actualitzaci del catleg
			cv = new ContentValues();
			cv.put(VER_DATA, sDataAct);	
			db.insert(TABLE_VER, null, cv);	
			
			db.setTransactionSuccessful();
		} finally {
			db.endTransaction();
		}
    }

    private ArrayList<Producte> obtenirNouCataleg() {
    	ArrayList<Producte> aProds   = new ArrayList<Producte>();
    	Producte			 oProd;
    	String result = "";
    	
    	try{
    		HttpClient 	httpclient = new DefaultHttpClient();
    	    HttpPost 	httppost   = new HttpPost(URL_CATALEG);
     	    HttpResponse response = httpclient.execute(httppost);
    	    HttpEntity entity = response.getEntity();
    	    InputStream is;
    	    
    	    if (entity != null) {
    	    	is = entity.getContent();
    	    	result = convertStreamToString(is);
    	    
	    	    JSONObject jsonResp = new JSONObject(result);
		    	JSONArray  jArray   = jsonResp.getJSONArray("prods");
		    	
		    	for (int iPos = 0; iPos < jArray.length(); iPos++) {
		    		oProd = new Producte();
		    		oProd.setNom(jArray.getJSONObject(iPos).getString("nom"));
		    		oProd.setDescripcio(jArray.getJSONObject(iPos).getString("descripcio"));
		    		aProds.add(oProd);
		    	}
    	    }
	    	
    	}catch(Exception e){
    	        Log.e("log_tag", "Error in http connection "+e.toString());
    	}
    	
    	return aProds;
    }
    
    public static String convertStreamToString(InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        StringBuilder sb = new StringBuilder();
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }

}
