package android.aLlum;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.util.Log;

public class Controler {
	
	private static Controler 	controler = null;
	
	// Array amb totes les comandes actives (no enviades)
	private ArrayList<Comanda> listCom;
	// Posici de la comanda actual
	private int					iPosAct;
	
	private Controler () {
		iPosAct = -1;	
	}
	 
	
	
	public static Controler getInstance(){
        if(controler == null){
        	controler = new Controler();
        }
        return controler;
    }

	public void inicialitzar(Context context) {
		BDComandes.getInstance().open(context);
	}
	
	public void finalitzar() {
		BDComandes.getInstance().close();
	}
	
	public void obtenirLlistaComandes() {
		// Obtenir la llista de comandes
		listCom = BDComandes.getInstance().obtenirLlistaComandes();
		
		// Afegir a la primera posici l'opci de 'Nova comanda'
		Comanda com = new Comanda();
		com.setId(-1);
		com.setNom("Nova comanda");
		listCom.add(0, com);
		
		// Carregar la llista de productes de cada comanda
		int iPos;
		for (iPos = 0; iPos < listCom.size(); iPos++) {
			Comanda c = (Comanda)listCom.get(iPos);
			// Obtenim la llista de productes 
			c.setLlistaProd(BDComandes.getInstance().obtenirLlistaProductes(c.getId()));
		}
		iPosAct = -1;
	}
	
	public String sincronitzar() {
		String           sMsg = "";
    	String 			 sServer, sLocal;
    	Date			 dServer, dLocal;
    	 
    	sServer = BDComandes.getInstance().obtenirDataVersioServidor();
    	if ( sServer == "") {
    		sMsg = "Error, no s'ha pogut connectar amb el servidor";
    	} else {
	    	sLocal  = BDComandes.getInstance().obtenirDataVersioLocal();
	
	    	Log.i("BDComandes", "dServidor: [" + sServer + "] dLocal: [" + sLocal + "]");
	     	
	    	if (sLocal.compareTo("") == 0) {
	    		sLocal = "2000-01-01 00:00:00";
	    	}
	    	
			dServer = ConvertToDate(sServer);
			dLocal  = ConvertToDate(sLocal);
			
		    // dServer > dLocal ==> iRes = 1
		    // dServer = dLocal ==> iRes = 0
		    // dServer < dLocal ==> iRes = -1
		    int iRes = dServer.compareTo(dLocal);
		    		
			if (iRes == 1) {
				BDComandes.getInstance().actualitzarCataleg(sServer);
				sMsg = "Versi sincronitzada amb dada: " + sServer;
			}
			else {
				sMsg = "La versi del catleg no ha canviat";
			}
    	}
    	
    	return sMsg;
    	
	}
	
	public ArrayList<Comanda> getListaComandes() {
		 return listCom;
	 }
	 
	public Comanda getComanda(int iPos) {
		iPosAct = iPos;
		return listCom.get(iPos);
	 }

	public Comanda getComanda() {
		return listCom.get(iPosAct);
	}
	
	public String getDescripcioProd() {
		String 		sDesc 	 = "";
		int			iNumProd = 0;
		int			iNumUnit = 0;
		int         iSumUnit = 0;
		int 		iPos  	 = 0;

		ArrayList<Producte> lProd = listCom.get(iPosAct).getLlistaProd();
		
		for (iPos = 0; iPos < lProd.size(); iPos++) {
			iNumUnit = lProd.get(iPos).getUnitats();
			if (iNumUnit > 0) {
				iNumProd++;
				iSumUnit += iNumUnit;
				sDesc += "  [" + String.format("%02d", iNumProd)  + "] " +
						 lProd.get(iPos).getNom() + " - " + iNumUnit + " unitats.\n";
			}
		}
		
		sDesc = "Nmero de productes = " + iNumProd + "\n" +
		        "Unitats totals = " + iSumUnit + "\n\n" + sDesc;
		return sDesc;
	}

	public Comanda getActComanda() {
        return listCom.get(iPosAct);
	}

	public Producte getProducte(int iPos) {
        return listCom.get(iPosAct).getLlistaProd().get(iPos);
	}

	public void setNomComanda (String sNom) {
		listCom.get(iPosAct).setNom(sNom);
	}
	
	public void setUnitats(int iPos, int iUnitats) {
		listCom.get(iPosAct).getLlistaProd().get(iPos).setUnitats(iUnitats);
	}

	public void desarComanda() {
		
		Comanda com = listCom.get(iPosAct);
		
		if (com.getId() > 0) {
			// Es tracta d'un update d'una comanda
			BDComandes.getInstance().actualitzarComanda(com.getId(), com.getNom());
			BDComandes.getInstance().actualitzarProdCom(com.getId(), com.getLlistaProd());
		} else {
			int idCom;
			
			// s una nova comanda
			idCom = BDComandes.getInstance().insertarComanda(com);
			com.setId(idCom);
			
			// Tornar a afegir l'opci de 'Nova comanda'
			Comanda newCom = new Comanda();
			newCom.setId(-1);
			newCom.setNom("Nova comanda");
			newCom.setLlistaProd(BDComandes.getInstance().obtenirLlistaProductes(-1));
			listCom.add(0, newCom);
		}
	}
	
	public void mostrarMissatge(Context context, String sTitol, String sMsg) {
    	AlertDialog dlg = new AlertDialog.Builder(context).create();
		dlg.setTitle(sTitol);
		dlg.setMessage(sMsg);
		dlg.setButton("OK", new DialogInterface.OnClickListener() {  
		      public void onClick(DialogInterface dialog, int which) {  
		        return;  
		    } }); 
		dlg.show();
    }
	
	private Date ConvertToDate(String dateString) {
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date 			 dRes;
		try {
			dRes = df.parse(dateString);
		   	Log.i("BDComandes", "data: [" + dRes.getDay() + 
			   	      "] [" + dRes.getMonth() + "] [" + dRes.getYear() +
			   	      "] [" + dRes.getHours() + "] [" + dRes.getMinutes() + "]");
			Log.i("BDComandes", "data2: [" + df.format(dRes) + "]");
		
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
		return dRes;
	}

}
